<?php

namespace Drupal\gift_aid_commerce\Event;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItemInterface;
use Drupal\Component\EventDispatcher\Event;
use Drupal\gift_aid\Entity\CharityInterface;

/**
 * Provides a event that allows listeners to modify how a gift aid declaration should be solicited for an order.
 */
class GiftAidCommerceOrderInfoEvent extends Event {

  /**
   * The order.
   *
   * @var \Drupal\commerce_order\Entity\OrderInterface
   */
  protected $order;

  /**
   * An array of order items eligible for Gift Aid.
   *
   * @var \Drupal\commerce_order\Entity\OrderItemInterface[]
   */
  protected $eligible = [];

  /**
   * Whether it is recommended to solicit Gift Aid declarations for this order.
   *
   * @var bool|null
   */
  protected $recommended = NULL;

  /**
   * The charity.
   *
   * @var \Drupal\gift_aid\Entity\CharityInterface
   */
  protected $charity;

  /**
   * Constructs a gift aid related contexts event object.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   */
  public function __construct(OrderInterface $order) {
    $this->order = $order;
  }

  /**
   * Gets the order.
   *
   * @return \Drupal\commerce_order\Entity\OrderInterface
   *   The order.
   */
  public function getOrder() {
    return $this->order;
  }

  /**
   * Gets the specified charity to solicit a Gift Aid declaration for.
   *
   * @return \Drupal\gift_aid\Entity\CharityInterface
   *   The charity.
   */
  public function getCharity() {
    return $this->charity;
  }

  /**
   * Specify the charity to solicit a Gift Aid declaration for.
   *
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   The charity.
   */
  public function setCharity(CharityInterface $charity) {
    $this->charity = $charity;
  }

  /**
   * Gets the order items that have been identified as eligible for Gift Aid.
   *
   * @return \Drupal\commerce_order\Entity\OrderItemInterface[]
   *   An array of order items eligible for Gift Aid.
   */
  public function getEligibleItems(): array {
    return $this->eligible;
  }

  /**
   * Specify which order items are eligible for Gift Aid.
   *
   * @param array $items
   *   The order items eligible for Gift Aid.
   */
  public function setEligibleItems(array $items) {
    if (!empty(array_filter($items, function ($item) {
        return !($item instanceof OrderItemInterface && $item->getOrderId() == $this->order->id());
    }))) {
      throw new \InvalidArgumentException('All provided entities must be order items from the current order.');
    }
    $this->eligible = $items;
  }

  /**
   * Specify that an order item is eligible for Gift Aid.
   *
   * @param \Drupal\commerce_order\Entity\OrderItemInterface $item
   *   The eligible order item.
   */
  public function addEligibleItem(OrderItemInterface $item) {
    if ($item->getOrderId() != $this->order->id()) {
      throw new \InvalidArgumentException('The provided item must from the current order.');
    }

    // Only add the item is it is not already specified as eligible.
    if (empty(array_filter($this->eligible, function ($existingItem) use ($item) {
      return $existingItem->id() == $item->id();
    }))) {
      $this->eligible[] = $item;
    }
  }

  /**
   * Determines whether one or more order items are eligible for Gift Aid.
   *
   * Usually there is no need to solicit a gift aid declaration if none of the present order items are eligible.
   *
   * @return bool
   *   Whether any order item is Gift Aid eligible.
   */
  public function hasEligible(): bool {
    return !empty($this->eligible);
  }

  /**
   * Determine whether it is recommended to solicit a Gift Aid declaration for this order.
   *
   * This defaults to the value of hasEligible(), unless setRecommended() has been called.
   *
   * @return bool|null
   *   Whether a declaration should be solicited. A null value indicates no opinion.
   */
  public function isRecommended(): ?bool {
    return $this->recommended;
  }

  /**
   * Specify whether it is recommended to solicit a Gift Aid declaration for this order.
   *
   * @param bool $recommended
   *   Whether a declaration should be solicited.
   */
  public function setRecommended($recommended) {
    $this->recommended = $recommended;
  }

}
