<?php

namespace Drupal\gift_aid_commerce\Plugin\Commerce\CheckoutPane;

use Drupal\commerce_checkout\Attribute\CommerceCheckoutPane;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutFlow\CheckoutFlowInterface;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutPane\CheckoutPaneBase;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\gift_aid_commerce\Event\GiftAidCommerceOrderInfoEvent;
use Drupal\gift_aid_commerce\Event\GiftAidCommerceEvents;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the gift aid information pane.
 */
#[CommerceCheckoutPane(
  id: "gift_aid_commerce",
  label: new TranslatableMarkup("Gift Aid declaration"),
  display_label: new TranslatableMarkup("Add Gift Aid"),
  default_step: "review",
)]
class GiftAidDeclaration extends CheckoutPaneBase {

  /**
   * The event that has information about how to solicit Gift Aid for this order.
   *
   * @var \Drupal\gift_aid_commerce\Event\GiftAidCommerceOrderInfoEvent
   */
  protected $giftAidOrderInfo;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The order service.
   *
   * @var \Drupal\gift_aid_commerce\OrderServiceInterface
   */
  protected $orderService;

  /**
   * Whether this order is already covered by a declaration.
   *
   * @var bool
   */
  protected $covered;

  /**
   * {@inheritdoc}
   */
  public function __construct(...$params) {
    parent::__construct(...$params);
    $this->buildOrderInfo();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, ?CheckoutFlowInterface $checkout_flow = NULL) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition, $checkout_flow);
    $instance->eventDispatcher = $container->get('event_dispatcher');
    $instance->orderService = $container->get('gift_aid_commerce.order_service');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function setOrder(OrderInterface $order) {
    parent::setOrder($order);
    $this->buildOrderInfo();
    return $this;
  }

  /**
   * Build Gift Aid order info.
   */
  protected function buildOrderInfo() {
    if ($this->order) {
      $this->giftAidOrderInfo = new GiftAidCommerceOrderInfoEvent($this->order);
      // Dependency injection isn't working for event dispatcher, not sure why.
      \Drupal::service('event_dispatcher')->dispatch($this->giftAidOrderInfo, GiftAidCommerceEvents::ORDER_INFO);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'show_when' => 'always',
      'show_if_eligible' => FALSE,
      'show_if_uncovered' => TRUE,
      'introduction' => $this->t("Increase your donation at no extra cost to you."),
      'call_to_action' => $this->t("Yes, I am a UK tax payer and would like Gift Aid claimed on my donations."),
      'explanation' => $this->t("I want to Gift Aid my donation and any donations I make in the future or have made in the past four years to {{ charity.label }}. I am a UK taxpayer and understand that if I pay less Income Tax and/or Capital Gains Tax than the amount of Gift Aid claimed on all my donations in that tax year it is my responsibility to pay any difference."),
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['show'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Collect Gift Aid declaration'),
    ];
    $form['show']['show_when'] = [
      '#title' => $this->t('When to collect Gift Aid declaration'),
      '#title_display' => 'invisible',
      '#type' => 'select',
      '#options' => [
        'always' => $this->t('Always'),
        'sometimes' => $this->t('Sometimes'),
      ],
      '#default_value' => $this->configuration['show_when'],
    ];
    $states = [
      'visible' => [
        ':input[name="configuration[panes][' . $this->getPluginId() . '][configuration][show][show_when]"]' => ['value' => 'sometimes'],
      ],
    ];
    $form['show']['show_if_eligible'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Only if the order contains a donation eligible for Gift Aid'),
      '#default_value' => $this->configuration['show_if_eligible'],
      '#states' => $states,
    ];
    $form['show']['show_if_uncovered'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Only if the order is not covered by a previous Gift Aid declaration'),
      '#default_value' => $this->configuration['show_if_uncovered'],
      '#states' => $states,
    ];

    $form['text'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Declaration text'),
    ];
    $form['text']['introduction'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Introduction'),
      '#default_value' => $this->configuration['introduction'],
    ];
    $form['text']['call_to_action'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Call to action'),
      '#default_value' => $this->configuration['call_to_action'],
    ];
    $form['text']['explanation'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Explanation'),
      '#default_value' => $this->configuration['explanation'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['show_when'] = $values['show']['show_when'];
      $this->configuration['show_if_eligible'] = $values['show']['show_if_eligible'];
      $this->configuration['show_if_uncovered'] = $values['show']['show_if_uncovered'];
      $this->configuration['introduction'] = $values['text']['introduction'];
      $this->configuration['call_to_action'] = $values['text']['call_to_action'];
      $this->configuration['explanation'] = $values['text']['explanation'];
    }
  }

  /**
   * Prepare text by substituting twig.
   *
   * @param $text
   *   The raw text.
   *
   * @return array
   *   An inline template render array.
   */
  protected function prepareTextRenderElement($text) {
    $context = [
      'commerce_order' => $this->order,
      'charity' => $this->giftAidOrderInfo->getCharity(),
    ];
    // Uses the inline template feature from Drupal to be able to use Twig.
    return [
      '#type' => 'inline_template',
      '#template' => $text,
      '#context' => $context,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildPaneForm(array $pane_form, FormStateInterface $form_state, array &$complete_form) {
    $pane_form['above'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ["gift-aid-commerce-declaration__above"]],
      'contents' => [
        'introduction' => [
          '#type' => 'container',
          'contents' => $this->prepareTextRenderElement($this->configuration['introduction']),
          '#attributes' => ['class' => ["gift-aid-commerce-declaration__introduction"]],
        ],
        'logo' => [
          '#markup' => '',
          '#prefix' => '<div class="gift-aid-commerce-declaration__logo">',
          '#suffix' => '</div',
        ],
      ],
    ];
    $preparedCta = $this->prepareTextRenderElement($this->configuration['call_to_action']);
    $pane_form['checkbox'] = [
      '#type' => 'checkbox',
      '#title' => \Drupal::service('renderer')->render($preparedCta),
      '#default_value' => FALSE,
      '#prefix' => '<div class="gift-aid-commerce-declaration__checkbox">',
      '#suffix' => '</div',
    ];
    $pane_form['explanation'] = [
      '#type' => 'container',
      'contents' => $this->prepareTextRenderElement($this->configuration['explanation']),
      '#attributes' => ['class' => ["gift-aid-commerce-declaration__notes"]],
    ];
    $pane_form['#attached']['library'][] = 'gift_aid_commerce/declaration';

    return $pane_form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitPaneForm(array &$pane_form, FormStateInterface $form_state, array &$complete_form) {
    // If the order is already placed, create the declaration now.
    // Otherwise, note it on the order for creation later when it is placed.
    // @see \Drupal\gift_aid_commerce\EventSubscriber\OrderPlaceSubscriber.
    $values = $form_state->getValues();
    $gift_aid_declared = !empty($values['gift_aid_commerce']['checkbox']);
    $this->order->setData('gift_aid_commerce_declared', $gift_aid_declared);
    if ($gift_aid_declared) {
      $this->order->setData('gift_aid_commerce_charity_id', $this->giftAidOrderInfo->getCharity()->id());
      $this->order->setData('gift_aid_commerce_explanation', $this->configuration['explanation']);

      if ($this->order->getState()->getId() !== 'draft') {
        $this->orderService->declare($this->order);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isVisible() {
    $visible = $this->hasCharity();
    if ($visible && $this->configuration['show_when'] == 'sometimes' && $this->giftAidOrderInfo) {
      $eligible = empty($this->configuration['show_if_eligible']) || $this->giftAidOrderInfo->hasEligible();
      $uncovered = empty($this->configuration['show_if_uncovered']) || !$this->isCoveredByDeclaration();
      $visible = $this->giftAidOrderInfo->isRecommended() ?? $eligible && $uncovered;
    }
    return $visible;
  }

  /**
   * Determine whether a charity has been identified for this order.
   *
   * @return bool
   *   Whether this order has a charity.
   */
  protected function hasCharity() {
    if ($this->giftAidOrderInfo) {
      $hasCharity = (bool) $this->giftAidOrderInfo->getCharity();
      if (!$hasCharity) {
        \Drupal::logger('gift_aid_commerce')->warning("Cannot collect Gift Aid declaration for order " . $this->order->id() . " as charity is not specified.");
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Determine whether this order is already covered by a declaration.
   *
   * @return bool
   *   Whether this order is already covered by a declaration.
   *
   * @throws \Exception
   */
  protected function isCoveredByDeclaration(): bool {
    if (!isset($this->covered)) {
      $storage = $this->entityTypeManager->getStorage('gift_aid_declaration');
      $this->covered = $storage->hasActiveByAnyContext($this->order, $this->giftAidOrderInfo->getCharity());
    }
    return $this->covered;
  }

}
