<?php

namespace Drupal\Tests\gift_aid_commerce\Kernel;

use Drupal\commerce_order\Entity\Order;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Tests\commerce_order\Kernel\OrderKernelTestBase;
use Drupal\Tests\gift_aid\Traits\GiftAidKernelTestTrait;

/**
 * Tests loading declarations in storage by context.
 *
 * @coversDefaultClass \Drupal\gift_aid\Declaration\DeclarationStorage
 * @group gift_aid_commerce
 */
class DeclarationStorageTest extends OrderKernelTestBase {

  use GiftAidKernelTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'datetime',
    'dynamic_entity_reference',
    'gift_aid',
    'gift_aid_commerce',
    'options',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->giftAidKernelSetUp();
  }

  /**
   *
   */
  protected function createOrder(array $values) {
    $defaults = [
      'type' => 'default',
      'store_id' => 'test',
      'state' => 'complete',
    ];
    $values = $values + $defaults;
    $order = Order::create($values);
    $order->save();
    return $order;
  }

  /**
   * Test the LoadByContext method.
   */
  public function testContexts() {
    $this->createDeclaration();
    $this->assertRelatedContextsCount($this->user, 1);
    $this->assertDeclarationsByAnyContextCount($this->user, 1);
    $this->assertRelatedContexts(TRUE, $this->user, $this->user);

    // If an order has a user customer, contexts for that order are found by that user and vice versa.
    $order1 = $this->createOrder([
      'uid' => $this->user->id(),
    ]);
    $this->assertRelatedContexts(TRUE, $this->user, $order1);
    $this->assertRelatedContexts(TRUE, $order1, $this->user);
    $this->assertRelatedContextsCount($this->user, 2);
    $this->assertDeclarationsByAnyContextCount($this->user, 1);
    $this->assertRelatedContextsCount($order1, 2);
    $this->assertDeclarationsByAnyContextCount($order1, 1);

    // Declarations found by order context.
    $this->createDeclaration([
      'context' => $order1,
    ]);
    $this->assertRelatedContextsCount($this->user, 2);
    $this->assertDeclarationsByAnyContextCount($this->user, 2);
    $this->assertRelatedContextsCount($order1, 2);
    $this->assertDeclarationsByAnyContextCount($order1, 2);

    // Multiple user declarations.
    $this->createDeclaration();
    $this->assertRelatedContextsCount($this->user, 2);
    $this->assertDeclarationsByAnyContextCount($this->user, 3);
    $this->assertRelatedContextsCount($order1, 2);
    $this->assertDeclarationsByAnyContextCount($order1, 3);

    // Draft, anonymous and other people's orders are ignored.
    return;
    // @todo Fix these once we decide about contexts
    $orderAnonymous = $this->createOrder([
      'uid' => 0,
    ]);
    $this->createDeclaration([
      'context' => $orderAnonymous,
    ]);
    $orderDraft = $this->createOrder([
      'uid' => $this->user->id(),
      'state' => 'draft',
    ]);
    $this->createDeclaration([
      'context' => $orderDraft,
    ]);
    $another = $this->createUser();
    $orderAnother = $this->createOrder([
      'uid' => $another->id(),
    ]);
    $this->createDeclaration([
      'context' => $orderAnother,
    ]);
    $this->assertRelatedContexts(FALSE, $this->user, $orderAnonymous);
    $this->assertRelatedContexts(FALSE, $this->user, $another);
    $this->assertRelatedContexts(FALSE, $this->user, $orderAnother);
    $this->assertRelatedContexts(FALSE, $order1, $orderAnother);
    $this->assertRelatedContexts(FALSE, $order1, $another);
    $this->assertRelatedContexts(FALSE, $order1, $orderAnother);
    $this->assertRelatedContextsCount($this->user, 2);
    $this->assertDeclarationsByAnyContextCount($this->user, 3);
    $this->assertRelatedContextsCount($order1, 2);
    $this->assertDeclarationsByAnyContextCount($order1, 3);

    // Multiple orders.
    $order2 = $this->createOrder([
      'uid' => $this->user->id(),
    ]);
    $this->assertRelatedContexts(TRUE, $this->user, $order2);
    $this->assertRelatedContexts(TRUE, $order1, $order2);
    $this->assertRelatedContextsCount($this->user, 3);
    $this->assertDeclarationsByAnyContextCount($this->user, 3);
    $this->assertRelatedContextsCount($order1, 3);
    $this->assertDeclarationsByAnyContextCount($order1, 3);
    $this->assertRelatedContextsCount($order2, 3);
    $this->assertDeclarationsByAnyContextCount($order2, 3);

    // Declarations by multiple order contexts.
    $this->createDeclaration([
      'context' => $order2,
    ]);
    $this->assertRelatedContextsCount($this->user, 3);
    $this->assertDeclarationsByAnyContextCount($this->user, 4);
    $this->assertRelatedContextsCount($order1, 3);
    $this->assertDeclarationsByAnyContextCount($order1, 4);
    $this->assertRelatedContextsCount($order2, 3);
    $this->assertDeclarationsByAnyContextCount($order2, 4);
  }

  /**
   *
   */
  protected function assertDeclarationsByAnyContextCount(EntityInterface $original_context, $count) {
    $storage = \Drupal::entityTypeManager()->getStorage('gift_aid_declaration');
    $declarations = $storage->loadByAnyContext($original_context);
    $this->assertCount($count, $declarations);
  }

  /**
   *
   */
  protected function assertRelatedContextsCount(EntityInterface $original_context, $count) {
    $contextsManager = \Drupal::service('gift_aid.related_contexts_manager');
    $contexts = $contextsManager->getRelatedContexts($original_context);
    $this->assertEquals($count, array_sum(array_map("count", $contexts)));
  }

  /**
   *
   */
  protected function assertRelatedContexts($expected, EntityInterface $original_context, $other_context) {
    $contextsManager = \Drupal::service('gift_aid.related_contexts_manager');
    $related = $contextsManager->isRelatedContext($original_context, $other_context);
    $this->assertSame($expected, $related);
  }

}
