<?php

namespace Drupal\gift_aid\Declaration;

use Drupal\Core\Entity\ContentEntityStorageInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\gift_aid\Entity\CharityInterface;

/**
 * Defines an interface for gift aid declaration storage.
 */
interface DeclarationStorageInterface extends ContentEntityStorageInterface {

  /**
   * Load gift aid declarations for a specific context.
   *
   * Optionally restrict declarations to a specific charity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $context
   *   The entity that is the context to find declarations for.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   (optional) The charity to find declarations for.
   *
   * @return \Drupal\gift_aid\Entity\DeclarationInterface[]
   *   An array of gift aid declaration objects indexed by their ids.
   */
  public function loadByContext(EntityInterface $context, ?CharityInterface $charity = NULL);

  /**
   * Load gift aid declarations for a specified set of contexts.
   *
   * Optionally restrict declarations to a specific charity.
   *
   * @param array $contexts
   *   An array keyed by entity type, where the values are arrays with keys and values that are entity ids of that type.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   (optional) The charity to find declarations for.
   *
   * @return \Drupal\gift_aid\Entity\DeclarationInterface[]
   *   An array of gift aid declaration objects indexed by their ids.
   */
  public function loadByContexts(array $contexts, ?CharityInterface $charity = NULL);

  /**
   * Load gift aid declarations for a specific context, and any related contexts.
   *
   * For example, if there is a declaration that has a commerce order as its context,
   * then this declaration should also be returned if the order customer (user) is
   * provided as the context to this method.
   *
   * Optionally restrict declarations to a specific charity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $original_context
   *   The entity that is the original context to find related contexts and declarations for.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   (optional) The charity to find declarations for.
   *
   * @return \Drupal\gift_aid\Entity\DeclarationInterface[]
   *   An array of gift aid declaration objects indexed by their ids.
   *
   * @see getContexts()
   */
  public function loadByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL);

  /**
   * Load ongoing gift aid declarations for a specific context, and any related contexts.
   *
   * @param \Drupal\Core\Entity\EntityInterface $original_context
   *   The entity that is the original context to find related contexts and declarations for.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   (optional) The charity to find declarations for.
   *
   * @return \Drupal\gift_aid\Entity\DeclarationInterface[]
   *   An array of gift aid declaration objects indexed by their ids.
   *
   * @see DeclarationInterface::isOngoing()
   */
  public function loadOngoingByAnyContext(EntityInterface $original_context, ?CharityInterface $charity = NULL);

  /**
   * Gets the active declaration for a context (or related contexts) and charity.
   *
   * If there are multiple ongoing date-based declarations for this charity in this context,
   * then get the one with no end date or the end date furthest in the future.
   *
   * @param \Drupal\Core\Entity\EntityInterface $original_context
   *   The entity that is the original context to find related contexts and declarations for.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   The charity to find declarations for.
   *
   * @return bool
   *   Whether there is an active declaration for this context, any related context, & charity.
   */
  public function getActiveByAnyContext(EntityInterface $original_context, CharityInterface $charity);

  /**
   * Determine whether an active declaration exists for a context (or related contexts) and charity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $original_context
   *   The entity that is the original context to find related contexts and declarations for.
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   The charity to find declarations for.
   *
   * @return bool
   *   Whether there is an active declaration for this context, any related context, & charity.
   */
  public function hasActiveByAnyContext(EntityInterface $original_context, CharityInterface $charity);

}
