<?php

namespace Drupal\gift_aid\Entity;

use CommerceGuys\Addressing\AddressFormat\AddressField;
use CommerceGuys\Addressing\AddressFormat\FieldOverride;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Charity entity.
 *
 * @ingroup gift_aid
 *
 * @ContentEntityType(
 *   id = "gift_aid_charity",
 *   label = @Translation("Charity"),
 *   plural_label = @Translation("Charities"),
 *   label = @Translation("Charity"),
 *   label_collection = @Translation("Charities"),
 *   label_singular = @Translation("charity"),
 *   label_plural = @Translation("charities"),
 *   label_count = @PluralTranslation(
 *     singular = "@count charity",
 *     plural = "@count charities"
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\gift_aid\Charity\CharityListBuilder",
 *     "storage" = "Drupal\gift_aid\Charity\CharityStorage",
 *     "form" = {
 *       "default" = "Drupal\gift_aid\Form\Charity\CharityForm",
 *       "add" = "Drupal\gift_aid\Form\Charity\CharityForm",
 *       "edit" = "Drupal\gift_aid\Form\Charity\CharityForm",
 *       "delete" = "Drupal\gift_aid\Form\Charity\CharityDeleteForm",
 *     },
 *     "access" = "Drupal\gift_aid\Charity\CharityAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\gift_aid\Charity\CharityHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "gift_aid_charity",
 *   data_table = "gift_aid_charity_field_data",
 *   translatable = TRUE,
 *   admin_permission = "administer gift aid charities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *   },
 *   links = {
 *     "add-form" = "/admin/gift-aid/charity/add",
 *     "edit-form" = "/admin/gift-aid/charity/{gift_aid_charity}/edit",
 *     "delete-form" = "/admin/gift-aid/charity/{gift_aid_charity}/delete",
 *     "collection" = "/admin/gift-aid/charity",
 *   },
 *   field_ui_base_route = "entity.gift_aid_charity.collection"
 * )
 */
class Charity extends ContentEntityBase implements CharityInterface {

  use EntityChangedTrait;

  // @todo implement EntityOwnerTrait.
  // @see https://www.drupal.org/project/drupal/issues/2949964

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRegisteredName() {
    return $this->get('registered_name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setRegisteredName($registered_name) {
    $this->set('registered_name', $registered_name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRegistrationNumber() {
    return $this->get('registration_number')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setRegistrationNumber($registration_number) {
    $this->set('registration_number', $registration_number);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRegisteredAddress() {
    return $this->get('registered_address')->first();
  }

  /**
   * {@inheritdoc}
   */
  public function setRegisteredAddress($registered_address) {
    $this->set('registered_address', $registered_address);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isDefault() {
    return (bool) $this->get('is_default')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setDefault($is_default = TRUE) {
    $this->set('is_default', (bool) $is_default);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name the charity is usually known by.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    // Registered Name field.
    $fields['registered_name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Registered Name'))
      ->setDescription(t('The Registered Name of the Charity.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -3,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    // Registration Number field.
    $fields['registration_number'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Registration Number'))
      ->setDescription(t('The Registration Number of the Charity.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -2,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -2,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    // Registered Address field.
    $fields['registered_address'] = BaseFieldDefinition::create('address')
      ->setLabel(t('Registered Address'))
      ->setDescription(t('The Registered Address of the Charity.'))
      ->setSetting('field_overrides', [
        AddressField::GIVEN_NAME => ['override' => FieldOverride::HIDDEN],
        AddressField::ADDITIONAL_NAME => ['override' => FieldOverride::HIDDEN],
        AddressField::FAMILY_NAME => ['override' => FieldOverride::HIDDEN],
        AddressField::ORGANIZATION => ['override' => FieldOverride::HIDDEN],
        AddressField::ADDRESS_LINE3 => ['override' => FieldOverride::HIDDEN],
      ])
      ->setDisplayOptions('form', [
        'type' => 'address_default',
        'weight' => -1,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    // 'default' is a reserved SQL word, hence the 'is_' prefix.
    $fields['is_default'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Default'))
      ->setDescription(t('Whether this is the default charity.'))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 10,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE);

    // Charity Author field.
    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Charity entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default');

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    /** @var \Drupal\gift_aid\Charity\CharityStorage $storage */
    parent::postSave($storage, $update);

    $default = $this->isDefault();
    $original_default = $this->original ? $this->original->isDefault() : FALSE;
    if ($default && !$original_default) {
      // The charity was set as default, remove the flag from other charities.
      $charity_ids = $storage->getQuery()
        ->condition('id', $this->id(), '<>')
        ->condition('is_default', TRUE)
        ->accessCheck(FALSE)
        ->execute();
      foreach ($charity_ids as $charity_id) {
        /** @var \Drupal\gift_aid\Entity\CharityInterface $charity */
        $charity = $storage->load($charity_id);
        $charity->setDefault(FALSE);
        $charity->save();
      }
    }
  }

}
