<?php

namespace Drupal\gift_aid\Entity;

use Drupal\Component\Utility\Html;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\DependencyInjection\ContainerNotInitializedException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\gift_aid\EntityValidationTrait;

/**
 * Defines the Declaration entity.
 *
 * @ingroup gift_aid
 *
 * @ContentEntityType(
 *   id = "gift_aid_declaration",
 *   label = @Translation("Gift Aid declaration"),
 *   label_collection = @Translation("Gift Aid declarations"),
 *   label_singular = @Translation("Gift Aid declaration"),
 *   label_plural = @Translation("Gift Aid declarations"),
 *   label_count = @PluralTranslation(
 *     singular = "@count Gift Aid declaration",
 *     plural = "@count Gift Aid declarations"
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\gift_aid\Declaration\DeclarationListBuilder",
 *     "storage" = "Drupal\gift_aid\Declaration\DeclarationStorage",
 *     "views_data" = "Drupal\gift_aid\Declaration\DeclarationViewsData",
 *     "form" = {
 *       "default" = "Drupal\gift_aid\Form\Declaration\DeclarationForm",
 *       "make" = "Drupal\gift_aid\Form\Declaration\DeclarationMakeForm",
 *       "delete" = "Drupal\gift_aid\Form\DeleteForm",
 *     },
 *     "access" = "Drupal\gift_aid\Declaration\DeclarationAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *       "revision" = "Drupal\Core\Entity\Routing\RevisionHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "gift_aid_declaration",
 *   data_table = "gift_aid_declaration_field_data",
 *   revision_table = "gift_aid_declaration_revision",
 *   revision_data_table = "gift_aid_declaration_field_revision",
 *   admin_permission = "administer gift aid declarations",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "revision_id",
 *     "uuid" = "uuid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log"
 *   },
 *   links = {
 *     "canonical" = "/admin/gift-aid/declaration/{gift_aid_declaration}",
 *     "add-form" = "/admin/gift-aid/declaration/add",
 *     "edit-form" = "/admin/gift-aid/declaration/{gift_aid_declaration}/edit",
 *     "delete-form" = "/admin/gift-aid/declaration/{gift_aid_declaration}/delete",
 *     "collection" = "/admin/gift-aid/declaration",
 *     "version-history" = "/admin/gift-aid/declaration/{gift_aid_declaration}/revisions",
 *     "revision" = "/admin/gift-aid/declaration/{gift_aid_declaration}/revisions/{gift_aid_declaration_revision}/view",
 *   },
 *   field_ui_base_route = "entity.gift_aid_declaration.collection",
 *   constraints = {
 *     "GiftAidDeclarationStartDate" = {}
 *   }
 * )
 */
class Declaration extends RevisionableContentEntityBase implements DeclarationInterface {

  use EntityValidationTrait {preSave as doValidate;
  }

  const STORAGE_TIMEZONE = 'Europe/London';

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    if (isset($values['declared_date']) && !isset($values['start_date'])) {
      $values['start_date'] = $values['declared_date'];
    }
    $today = static::today();
    $values += [
      'uid' => \Drupal::currentUser()->id(),
      'start_date' => $today,
      'declared_date' => $today,
      'date_based' => TRUE,
      'charity' => \Drupal::entityTypeManager()->getStorage('gift_aid_charity')->loadDefault(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function preDelete(EntityStorageInterface $storage, array $entities) {
    parent::preDelete($storage, $entities);

    /** @var \Drupal\gift_aid\Entity\Declaration $declaration */
    foreach ($entities as $declaration) {
      if ($evidence = $declaration->getEvidence()) {
        $evidence->delete();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    if (!$this->isDateBased()) {
      $this->setStartDate(NULL);
      $this->setEndDate(NULL);
    }

    parent::preSave($storage);
    $this->doValidate($storage);
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return Html::escape("{$this->getDonor()->label()} {$this->getDeclaredDate(TRUE)} {$this->getEvidence()->type->entity->label()} ({$this->id()})");
  }

  /**
   * {@inheritdoc}
   */
  public function getDonor() {
    return $this->get('donor')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setDonor(DonorInterface $donor) {
    $this->set('donor', $donor->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getDonorId() {
    return $this->get('donor')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setDonorId(int $donor_id) {
    $this->set('donor', $donor_id);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCharity() {
    return $this->get('charity')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setCharity(CharityInterface $charity) {
    $this->set('charity', $charity->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCharityId() {
    return $this->get('charity')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setCharityId(int $charity_id) {
    $this->set('charity', $charity_id);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getEvidence() {
    return $this->get('evidence')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setEvidence(EvidenceInterface $evidence) {
    $this->set('evidence', $evidence->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getExplanation() {
    return $this->get('explanation')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setExplanation(string $explanation) {
    $this->set('explanation', $explanation);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeclaredDate(bool $formatted = FALSE) {
    return $this->getDateField('declared_date', $formatted);
  }

  /**
   * {@inheritdoc}
   */
  public function setDeclaredDate($date) {
    return $this->setDateField('declared_date', $date);
  }

  /**
   * {@inheritdoc}
   */
  public function hasStarted() {
    $start = $this->getStartDate(TRUE);
    if (!$this->isDateBased() || !$start) {
      return FALSE;
    }
    return ($start <= static::today());
  }

  /**
   * {@inheritdoc}
   */
  public function getStartDate(bool $formatted = FALSE) {
    return $this->getDateField('start_date', $formatted, [0, 0, 0]);
  }

  /**
   * {@inheritdoc}
   */
  public function setStartDate($date) {
    return $this->setDateField('start_date', $date);
  }

  /**
   * {@inheritdoc}
   */
  public function hasEnded() {
    $end = $this->getEndDate(TRUE);
    if (!$this->isDateBased() || !$end) {
      return FALSE;
    }
    return ($end < static::today());
  }

  /**
   * {@inheritdoc}
   */
  public function getEndDate(bool $formatted = FALSE) {
    return $this->getDateField('end_date', $formatted, [23, 59, 59]);
  }

  /**
   * {@inheritdoc}
   */
  public function hasEndDate() {
    return !$this->get('end_date')->isEmpty();
  }

  /**
   * {@inheritdoc}
   */
  public function setEndDate($date) {
    return $this->setDateField('end_date', $date);
  }

  /**
   * {@inheritdoc}
   */
  public function cancel() {
    return $this->setEndDate(static::today());
  }

  /**
   * {@inheritdoc}
   */
  public function isConfirmationSent() {
    return !$this->get('confirmation_date')->isEmpty();
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmationDate(bool $formatted = FALSE) {
    return $this->getDateField('confirmation_date', $formatted);
  }

  /**
   * {@inheritdoc}
   */
  public function setConfirmationDate($date) {
    return $this->setDateField('confirmation_date', $date);
  }

  /**
   * {@inheritdoc}
   */
  public function getClaimableDate(bool $formatted = FALSE) {
    return $this->getDateField('claimable_date', $formatted);
  }

  /**
   * {@inheritdoc}
   */
  public function isClaimable() {
    $date = $this->getClaimableDate(TRUE);
    return $date && ($date <= static::today());
  }

  /**
   * {@inheritdoc}
   */
  public function setClaimableDate($date) {
    return $this->setDateField('claimable_date', $date);
  }

  /**
   * {@inheritdoc}
   */
  public function isValid() {
    return $this->getValidity() == DeclarationInterface::DECLARATION_VALID;
  }

  /**
   * {@inheritdoc}
   */
  public function isConfirmationMissing() {
    return $this->getValidity() == DeclarationInterface::DECLARATION_CONFIRMATION_MISSING;
  }

  /**
   * {@inheritdoc}
   */
  public function getValidity() {
    return $this->get('validity')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setValidity(string $valid) {
    $this->set('validity', $valid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus() {
    if (!$this->isValid()) {
      return $this->getValidity();
    }
    elseif (!$this->isClaimable()) {
      return DeclarationInterface::DECLARATION_PROVISIONAL;
    }
    elseif (!$this->isDateBased()) {
      return DeclarationInterface::DECLARATION_SELECTIVE;
    }
    elseif ($this->hasEnded()) {
      return DeclarationInterface::DECLARATION_ENDED;
    }
    elseif ($this->hasStarted()) {
      return DeclarationInterface::DECLARATION_ONGOING;
    }
    else {
      return DeclarationInterface::DECLARATION_PENDING;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getStatusString() {
    return static::statusOptions()[$this->getStatus()];
  }

  /**
   * {@inheritdoc}
   */
  public function isOngoing() {
    return $this->isValid() && $this->hasStarted() && !$this->hasEnded();
  }

  /**
   * {@inheritdoc}
   */
  public function isDateBased() {
    return !$this->get('date_based')->isEmpty() && $this->get('date_based')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setDateBased($status) {
    $this->set('date_based', (bool) $status);
    return $this;
  }

  /**
   * Gets a date field.
   *
   * @param string $fieldName
   *   Name of the field to get.
   * @param bool $formatted
   *   TRUE to return the date as an ISO format string.
   * @param array $time
   *   (optional) Time to set as an array of hour, minute, second.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string|null
   *   Date as an object or string or null if the date isn't set.
   */
  protected function getDateField(string $fieldName, bool $formatted, array $time = []) {
    $value = $this->get($fieldName)->value;
    if ($formatted || is_null($value)) {
      return $value;
    }
    $date = DrupalDateTime::createFromFormat(DateTimeItemInterface::DATE_STORAGE_FORMAT, $value, static::STORAGE_TIMEZONE);
    if ($time) {
      $date->setTime(...$time);
    }
    return $date;
  }

  /**
   * Sets a date field.
   *
   * @param string $fieldName
   *   Name of the field to set.
   * @param \Drupal\Core\Datetime\DrupalDateTime|string|null $date
   *   The date as an object or string (in ISO format) or NULL.
   *
   * @return $this
   */
  protected function setDateField(string $fieldName, $date) {
    if ($date instanceof DrupalDateTime) {
      $date->setTimezone(new \DateTimeZone(static::STORAGE_TIMEZONE));
      $date = $date->format(DateTimeItemInterface::DATE_STORAGE_FORMAT);
    }
    $this->set($fieldName, $date);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function validityOptions() {
    return [
      DeclarationInterface::DECLARATION_INVALID => t('Invalid'),
      DeclarationInterface::DECLARATION_CONFIRMATION_MISSING => t('Written confirmation missing'),
      DeclarationInterface::DECLARATION_VALID => t('Valid'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function statusOptions() {
    return static::validityOptions() + [
      DeclarationInterface::DECLARATION_PROVISIONAL => t('Provisional'),
      DeclarationInterface::DECLARATION_SELECTIVE => t('Selective'),
      DeclarationInterface::DECLARATION_PENDING => t('Pending'),
      DeclarationInterface::DECLARATION_ONGOING => t('Ongoing'),
      DeclarationInterface::DECLARATION_ENDED => t('Ended'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['donor'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Donor'))
      ->setSetting('target_type', 'gift_aid_donor')
      ->setDescription(t('The donor for the declaration.'))
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', ['weight' => 0])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['charity'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Charity'))
      ->setDescription(t('The charity the declaration is for.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'gift_aid_charity')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['evidence'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Evidence'))
      ->setDescription(t('The evidence for the declaration.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'gift_aid_evidence')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_entity_view',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
        'weight' => 0,
        'settings' => [
          'removed_reference' => 'delete',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['explanation'] = BaseFieldDefinition::create('string')
      ->setSetting('max_length', 1000)
      ->setLabel(t('Explanation'))
      ->setDescription(t('Explanation shown to the donor of the personal tax implications associated with making a Gift Aid donation.'))
      ->setDisplayOptions('form', ['weight' => 0])
      ->setDisplayOptions('view', ['weight' => 0])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['status'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Status'))
      ->setDescription(t('The status of the declaration.'))
      ->setComputed(TRUE)
      ->setClass('\Drupal\gift_aid\Plugin\Field\DeclarationStatusItemList')
      ->setSetting('allowed_values_function', [static::class, 'statusOptions'])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['declared_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Declared date'))
      ->setDescription(t('The date the declaration was made by the donor.'))
      ->setRevisionable(TRUE)
      ->setDefaultValue([
        'default_date_type' => 'now',
        'default_date' => 'now',
      ])
      ->setSetting('datetime_type', DateTimeItem::DATETIME_TYPE_DATE)
      ->setDisplayOptions('view', [
        'type' => 'datetime_default',
        'label' => 'inline',
        'weight' => 0,
        'settings' => [
          'format_type' => 'html_date',
        ],
      ])
      ->setDisplayOptions('form', ['weight' => 0])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['date_based'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Date-based'))
      ->setDescription(t('Is this a date-based declaration that covers all donations within a period of time?'))
      ->setRevisionable(TRUE)
      ->setDefaultValue(TRUE)
      ->setSettings([
        'on_label' => t('Yes'),
        'off_label' => t('No'),
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', ['weight' => 0])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['start_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Start date'))
      ->setDescription(t('Only donations made after this date are covered by the declaration.'))
      ->setRevisionable(TRUE)
      ->setDefaultValue([
        'default_date_type' => 'now',
        'default_date' => 'now',
      ])
      ->setSetting('datetime_type', DateTimeItem::DATETIME_TYPE_DATE)
      ->setDisplayOptions('view', [
        'type' => 'datetime_default',
        'label' => 'inline',
        'weight' => 0,
        'settings' => [
          'format_type' => 'html_date',
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['end_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('End date'))
      ->setDescription(t('Donations made after this date are not covered by the declaration.'))
      ->setRevisionable(TRUE)
      ->setSetting('datetime_type', DateTimeItem::DATETIME_TYPE_DATE)
      ->setDisplayOptions('view', [
        'type' => 'datetime_default',
        'label' => 'inline',
        'weight' => 0,
        'settings' => [
          'format_type' => 'html_date',
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $valid_description = t('A valid declaration must: <ul><li>state the donor’s full name and home address</li><li>name the charity</li><li>identify the gift or ifts to which the declaration relates (for example, a particular donation or all donations)</li><li>confirm that the idenified gift or gifts are to be treated as Gift Aid donations</li><li>demonstrate that the Charity has given an adequate explanation to the donor of the personal tax implications</li></ul>');

    $fields['validity'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Validity'))
      ->setDescription($valid_description)
      ->setRevisionable(TRUE)
      ->setSetting('allowed_values', static::validityOptions())
      ->setDefaultValue(DeclarationInterface::DECLARATION_INVALID)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_buttons',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['confirmation_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Written confirmation sent date'))
      ->setDescription(t('The date a first written confirmation of the declaration was sent to the donor. This is required for oral declarations, and can also be used as part of a process of validating declarations that were invalid due to insufficient auditable information.'))
      ->setRevisionable(TRUE)
      ->setSetting('datetime_type', 'date')
      ->setDisplayOptions('view', [
        'type' => 'datetime_default',
        'settings' => [
          'format_type' => 'html_date',
        ],
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['claimable_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Claimable date'))
      ->setDescription(t('The date when the declaration is no longer at risk of retraction, so can be claimed with HMRC.'))
      ->setRevisionable(TRUE)
      ->setSetting('datetime_type', 'date')
      ->setDisplayOptions('view', [
        'type' => 'datetime_default',
        'label' => 'inline',
        'weight' => 0,
        'settings' => [
          'format_type' => 'html_date',
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields += static::revisionLogBaseFieldDefinitions($entity_type);
    $fields['revision_log']->setLabel(t('Log message'));

    return $fields;
  }

  /**
   * Gets a date string for today.
   *
   * @return string
   *   Date string for today.
   */
  public static function today() {
    // Prevent weird errors in Declaration::getStatus() in test failures when trying to
    // serialise a declaration entity without a container.
    try {
      \Drupal::getContainer();
      $now = \Drupal::time()->getCurrentTime();
    }
    catch (ContainerNotInitializedException $e) {
      $now = 0;
    }

    $date = DrupalDateTime::createFromTimestamp($now, static::STORAGE_TIMEZONE);
    return $date->format(DateTimeItemInterface::DATE_STORAGE_FORMAT);
  }

}
