<?php

namespace Drupal\gift_aid\Entity;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Provides an interface for defining declarations.
 *
 * @ingroup gift_aid
 */
interface DeclarationInterface extends ContentEntityInterface {

  // Declaration validity.
  const DECLARATION_INVALID = 'invalid';
  const DECLARATION_CONFIRMATION_MISSING = 'confirmation missing';
  const DECLARATION_VALID = 'valid';

  // Declaration status constants (in addition to the validity values).
  const DECLARATION_PROVISIONAL = 'provisional';
  const DECLARATION_SELECTIVE = 'limited';
  const DECLARATION_PENDING = 'pending';
  const DECLARATION_ONGOING = 'ongoing';
  const DECLARATION_ENDED = 'ended';

  /**
   * Gets the Donor of the Declaration.
   *
   * @return \Drupal\gift_aid\Entity\DonorInterface
   *   The Donor entity.
   */
  public function getDonor();

  /**
   * Sets the Donor entity.
   *
   * @param \Drupal\gift_aid\Entity\DonorInterface $donor
   *   Donor entity.
   *
   * @return $this
   */
  public function setDonor(DonorInterface $donor);

  /**
   * Gets the Donor ID of the Declaration.
   *
   * @return int
   *   The Donor entity ID.
   */
  public function getDonorId();

  /**
   * Sets the Donor entity ID of the declaration.
   *
   * @param int $donor_id
   *   Donor entity ID.
   *
   * @return $this
   */
  public function setDonorId(int $donor_id);

  /**
   * Gets the Charity of the Declaration.
   *
   * @return \Drupal\gift_aid\Entity\CharityInterface
   *   The Charity entity.
   */
  public function getCharity();

  /**
   * Sets the Charity entity.
   *
   * @param \Drupal\gift_aid\Entity\CharityInterface $charity
   *   Charity entity.
   *
   * @return $this
   */
  public function setCharity(CharityInterface $charity);

  /**
   * Gets the Charity ID of the Declaration.
   *
   * @return int
   *   The Charity entity ID.
   */
  public function getCharityId();

  /**
   * Sets the Charity entity ID of the declaration.
   *
   * @param int $charity_id
   *   Charity entity ID.
   *
   * @return $this
   */
  public function setCharityId(int $charity_id);

  /**
   * Gets the Evidence of the declaration.
   *
   * @return \Drupal\gift_aid\Entity\EvidenceInterface
   *   The Evidence entity.
   */
  public function getEvidence();

  /**
   * Sets the Evidence of the declaration.
   *
   * @param \Drupal\gift_aid\Entity\EvidenceInterface $evidence
   *   Evidence entity.
   *
   * @return $this
   */
  public function setEvidence(EvidenceInterface $evidence);

  /**
   * Gets the explanation message that was shown to the declarer.
   *
   * @return string
   *   Explanation message shown to declarer about their responsibilities.
   */
  public function getExplanation();

  /**
   * Sets the explanation message that was shown to the declarer.
   *
   * @param string $explanation
   *   Explanation message shown to declarer about their responsibilities.
   *
   * @return $this
   */
  public function setExplanation(string $explanation);

  /**
   * Gets the date the declaration was declared.
   *
   * @param bool $formatted
   *   (optional) TRUE to return the date as an ISO format string.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string
   *   Declared date as an object or formatted string.
   */
  public function getDeclaredDate(bool $formatted = FALSE);

  /**
   * Sets the declared date.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime|string $date
   *   The declared date as an object or string (in ISO format).
   *
   * @return $this
   */
  public function setDeclaredDate($date);

  /**
   * Whether or not a date-based declaration has taken effect.
   *
   * @return bool
   *   Whether the start date has passed.
   */
  public function hasStarted();

  /**
   * Gets the start date of the period covered by this declaration.
   *
   * @param bool $formatted
   *   (optional) TRUE to return the date as an ISO format string.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string|null
   *   Start date as an object or string or NULL if the start date isn't set.
   */
  public function getStartDate(bool $formatted = FALSE);

  /**
   * Sets the start date.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime|string|null $date
   *   The start date as an object or string (in ISO format) or NULL.
   *
   * @return $this
   */
  public function setStartDate($date);

  /**
   * Whether or not the gift aid declaration has ended.
   *
   * @return bool
   *   Whether the end date has passed.
   */
  public function hasEnded();

  /**
   * Gets the end date of the period covered by this declaration.
   *
   * @param bool $formatted
   *   (optional) TRUE to return the date as an ISO format string.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string|null
   *   End date as an object or string or null if the end date isn't set.
   */
  public function getEndDate(bool $formatted = FALSE);

  /**
   * Whether the declaration has an end date specified.
   *
   * @return bool
   *   Whether the declaration has an end date.
   */
  public function hasEndDate();

  /**
   * Sets the end date.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime|string|null $date
   *   The end date as an object or string (in ISO format) or NULL.
   *
   * @return $this
   */
  public function setEndDate($date);

  /**
   * Cancels the declaration by setting the end date to today.
   *
   * @return $this
   */
  public function cancel();

  /**
   * Whether a written confirmation of this declaration has been sent to the declarer.
   *
   * @return bool
   *   Whether a written confirmation has been sent.
   */
  public function isConfirmationSent();

  /**
   * Gets the date a written confirmation was sent.
   *
   * @param bool $formatted
   *   (optional) TRUE to return the date as an ISO format string.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string|null
   *   Written confirmation date as an object or string or null if the end date isn't set.
   */
  public function getConfirmationDate();

  /**
   * Sets the date a written confirmation was sent.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime|string|null $date
   *   The written confirmation date as an object or string (in ISO format) or NULL.
   *
   * @return $this
   */
  public function setConfirmationDate(string $date);

  /**
   * Gets the date when the declaration can safely be used to make a claim.
   *
   * @param bool $formatted
   *   (optional) TRUE to return the date as an ISO format string.
   *
   * @return \Drupal\Core\Datetime\DrupalDateTime|string|null
   *   Claimable date as an object or string or null if the date isn't set.
   */
  public function getClaimableDate(bool $formatted = FALSE);

  /**
   * Gets whether the declaration can safely be used to make a claim.
   *
   * @return bool
   *   Whether the declaration is claimable.
   */
  public function isClaimable();

  /**
   * Gets the date when the declaration can safely be used to make a claim.
   *
   * @param \Drupal\Core\Datetime\DrupalDateTime|string|null $date
   *   The claimable date as an object or string (in ISO format) or NULL.
   *
   * @return $this
   */
  public function setClaimableDate($date);

  /**
   * Whether the declaration is valid.
   *
   * Declarations are invalid if they lack sufficient auditable information.
   */
  public function isValid();

  /**
   * Whether a written confirmation is required and missing for this declaration.
   *
   * @return bool
   *   Whether a written confirmation is missing.
   */
  public function isConfirmationMissing();

  /**
   * Gets the validity.
   *
   * @return string
   *   The validity: DECLARATION_INVALID, DECLARATION_CONFIRMATION_MISSING or
   *   DECLARATION_VALID.
   */
  public function getValidity();

  /**
   * Sets the validity.
   *
   * @param string $valid
   *   The validity: DECLARATION_INVALID, DECLARATION_CONFIRMATION_MISSING or
   *   DECLARATION_VALID.
   *
   * @return $this
   */
  public function setValidity(string $valid);

  /**
   * Gets the declaration status.
   *
   * @return string
   *   The status.
   */
  public function getStatus();

  /**
   * Whether or not the declaration is date-based, applying to any donation between the start and end dates.
   *
   * @return bool
   *   Whether the declaration is date-based.
   */
  public function isDateBased();

  /**
   * Set whether the declaration is date-based.
   *
   * @param bool $date_based
   *   Whether the declaration is date_based.
   *
   * @return $this
   */
  public function setDateBased($date_based);

}
