<?php

namespace Drupal\Tests\gift_aid\FunctionalJavascript;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Url;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\gift_aid\Entity\DeclarationInterface;
use Drupal\Tests\gift_aid\Traits\GiftAidTestTrait;

/**
 * Tests the Gift Aid declaration create and edit forms.
 *
 * @group gift_aid
 */
class DeclarationFormTest extends WebDriverTestBase {

  use GiftAidTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'gift_aid',
    'gift_aid_user',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->charity = $this->createCharity(['is_default' => TRUE]);
  }

  /**
   * Tests a logged in user making a web declaration.
   */
  public function testWebDeclarationMadeByDonor() {
    // A donor makes a declaration.
    $donor_user = $this->createUser(['make gift aid declaration']);
    $this->drupalLogin($donor_user);
    $url = Url::fromRoute('gift_aid.user_context', ['user' => $donor_user->id()]);
    $this->drupalGet($url->toString());
    $this->assertSession()->pageTextContains('I am a UK taxpayer');
    $edit = [
      'address[0][address][given_name]' => $this->randomString(),
      'address[0][address][family_name]' => $this->randomString(),
      'address[0][address][address_line1]' => $this->randomString(),
      'address[0][address][locality]' => $this->randomString(),
    ];
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('Thank you');

    // There should now be a stored donor and declaration.
    $donor = \Drupal::entityTypeManager()->getStorage('gift_aid_donor')->loadOrCreateByContext($donor_user);
    $this->assertFalse($donor->isNew());
    $this->assertEquals('user', $donor->getContext()->getEntityTypeId());
    $declarations = \Drupal::entityTypeManager()->getStorage('gift_aid_declaration')->loadByContext($donor_user);
    $declaration = reset($declarations);
    $this->assertInstanceOf(DeclarationInterface::class, $declaration);
    $this->assertEquals('user', $declaration->getEvidence()->bundle());
    $this->assertEquals($this->charity->id(), $declaration->getCharity()->id());
    $this->assertTrue($declaration->isDateBased());
    $this->assertFalse($declaration->hasEndDate());
    $this->assertTrue($declaration->isOngoing());
    $this->assertTrue($declaration->isClaimable());
    $this->assertEquals($donor->id(), $declaration->getDonorId());

    // @todo Fix when added code for cancellation.
    return;
    // The donor can cancel the declaration.
    $date_format = DateFormat::load('html_date')->getPattern();
    $this->drupalGet($declaration->toUrl('edit')->toString());
    $now = \Drupal::time()->getCurrentTime();
    $today = DrupalDateTime::createFromTimestamp($now, DateTimeItemInterface::STORAGE_TIMEZONE);
    $this->assertSession()->fieldValueEquals("date_ended[0][value][date]", $today->format($date_format));

    // Entering a date in the past should produce a validation error and not save.
    $past_date = new DrupalDateTime('2012-12-31 00:00:00', DateTimeItemInterface::STORAGE_TIMEZONE);
    $edit = ["date_ended[0][value][date]" => $past_date->format($date_format)];
    $this->submitForm($edit, t('Cancel'));
    $this->assertSession()->pageTextContains("Please enter an end date that is not in the past.");
    $declaration = $this->reloadDeclaration($declaration);
    $this->assertFalse($declaration->hasEndDate());

    $edit = ["date_ended[0][value][date]" => $today->format($date_format)];
    $this->submitForm($edit, t('Cancel'));
    $this->assertSession()->pageTextNotContains("Please enter an end date that is not in the past.");
    $declaration = $this->reloadDeclaration($declaration);
    $this->assertSame($this->charity->id(), $declaration->getCharity()->id());
    $this->assertSame('user', $declaration->getContext()->getEntityTypeId());
    $this->assertSame($donor->id(), $declaration->getContext()->getEntityTypeId());
    $this->assertTrue($declaration->isDateBased());
    $this->assertTrue($declaration->hasEndDate());
    $this->assertFalse($declaration->isOngoing());
  }

  /**
   * Tests staff recording a declaration for someone else.
   */
  public function testOralDeclarationRecordedByStaff() {
    // Staff makes a declaration.
    $donor_user = $this->createUser();
    $staff = $this->createUser(['record gift aid declarations made by others']);
    $this->drupalLogin($staff);

    $url = Url::fromRoute('gift_aid.user_context', ['user' => $donor_user->id()]);
    $this->drupalGet($url->toString());
    $this->assertSession()->pageTextContains('Gift Aid is not currently active');
    $edit = [
      'address[0][address][given_name]' => $this->randomString(),
      'address[0][address][family_name]' => $this->randomString(),
      'address[0][address][address_line1]' => $this->randomString(),
      'address[0][address][locality]' => $this->randomString(),
    ];
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextNotContains('Thank you');
    $this->assertSession()->pageTextContains('There are no Gift Aid declarations yet.');

    // There should now be a stored donor.
    $donor = \Drupal::entityTypeManager()->getStorage('gift_aid_donor')->loadOrCreateByContext($donor_user);
    $this->assertFalse($donor->isNew());

    // Add a donation.
    $this->clickLink('Add Declaration');
    $edit = [
      'explanation' => $this->randomString(),
    ];
    // @todo Use ajax to add evidence.
    return;
    $this->submitForm($edit, 'Save');

    // There should now be a stored declaration.
    $declarations = \Drupal::entityTypeManager()->getStorage('gift_aid_declaration')->loadByContext($donor_user);
    $declaration = reset($declarations);
    $this->assertInstanceOf(DeclarationInterface::class, $declaration);
    $this->assertSame($this->charity->id(), $declaration->getCharity()->id());
    $this->assertSame('user', $declaration->getContext()->getEntityTypeId());
    $this->assertSame($donor->id(), $declaration->getContext()->getEntityTypeId());
    $this->assertTrue($declaration->isDateBased());
    $this->assertFalse($declaration->hasEndDate());
    $this->assertTrue($declaration->isOngoing());
    $this->assertSame($staff->id(), $declaration->getOwnerId());

    // @todo Fix when added code for cancellation.
    return;
    // The donor can cancel the declaration.
    $date_format = DateFormat::load('html_date')->getPattern();
    $this->drupalGet($declaration->toUrl('edit')->toString());
    $now = \Drupal::time()->getCurrentTime();
    $today = DrupalDateTime::createFromTimestamp($now, DateTimeItemInterface::STORAGE_TIMEZONE);
    $this->assertSession()->fieldValueEquals("date_ended[0][value][date]", $today->format($date_format));

    // Entering a date in the past should produce a validation error and not save.
    $past_date = new DrupalDateTime('2012-12-31 00:00:00', DateTimeItemInterface::STORAGE_TIMEZONE);
    $edit = ["date_ended[0][value][date]" => $past_date->format($date_format)];
    $this->submitForm($edit, t('Cancel'));
    $this->assertSession()->pageTextContains("Please enter an end date that is not in the past.");
    $declaration = $this->reloadDeclaration($declaration);
    $this->assertFalse($declaration->hasEndDate());

    $edit = ["date_ended[0][value][date]" => $today->format($date_format)];
    $this->submitForm($edit, t('Cancel'));
    $this->assertSession()->pageTextNotContains("Please enter an end date that is not in the past.");
    $declaration = $this->reloadDeclaration($declaration);
    $this->assertSame($this->charity->id(), $declaration->getCharity()->id());
    $this->assertSame('user', $declaration->getContext()->getEntityTypeId());
    $this->assertSame($donor->id(), $declaration->getContext()->getEntityTypeId());
    $this->assertTrue($declaration->isDateBased());
    $this->assertTrue($declaration->hasEndDate());
    $this->assertFalse($declaration->isOngoing());
  }

  /**
   *
   */
  protected function reloadDeclaration(DeclarationInterface $declaration) {
    $storage = \Drupal::entityTypeManager()->getStorage('gift_aid_declaration');
    $storage->resetCache([$declaration->id()]);
    return $storage->load($declaration->id());
  }

}
