<?php

namespace Drupal\Tests\gift_aid\Kernel;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\gift_aid\Entity\Declaration;
use Drupal\gift_aid\Entity\DeclarationInterface;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\user\UserInterface;

/**
 * Tests the declaration entity.
 *
 * @coversDefaultClass \Drupal\gift_aid\Entity\Declaration
 * @group gift_aid
 */
class DeclarationTest extends GiftAidTestBase {

  use NodeCreationTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'node',
  ];

  /**
   * Tests CRUD operations.
   *
   * @covers ::getCharityId
   * @covers ::getDeclaredDate
   * @covers ::getStartDate
   * @covers ::getEndDate
   * @covers ::getCancelledDate
   * @covers ::getStatus
   * @covers ::isDateBased
   * @covers ::getContext
   * @covers ::setCharity
   * @covers ::setDeclaredDate
   * @covers ::setStartDate
   * @covers ::setEndDate
   * @covers ::setCancelledDate
   * @covers ::setDeclarationStatus
   * @covers ::setDateBased
   * @covers ::setContext
   */

  /**
   * Test the creation of declaration.
   */
  public function testDefaults() {
    $declaration = $this->createDeclaration();

    $today = Declaration::today();
    $this->assertTrue($declaration instanceof Declaration, 'The declaration with date-based is created.');
    $this->assertEquals($this->charity->id(), $declaration->getCharityId(), 'Testing declaration charity.');
    $this->assertEquals($today, $declaration->getDeclaredDate(TRUE), "Declared date should default to current date.");
    $this->assertEquals($today, $declaration->getStartDate(TRUE), "Start date should default to current date.");
    $this->assertNull($declaration->getConfirmationDate(), 'Testing written statement sent date.');
    $this->assertFalse($declaration->isClaimable(), 'Testing claimable.');
    $this->assertTrue($declaration->hasStarted(), 'Testing started.');
    $this->assertFalse($declaration->hasEnded(), 'Testing ended.');
    $this->assertTrue($declaration->isValid(), 'Testing valid.');
    $this->assertTrue($declaration->isOngoing(), 'Testing ongoing.');
    $this->assertEquals(DeclarationInterface::DECLARATION_PROVISIONAL, $declaration->getStatus(), 'Testing declaration status.');
    $this->assertTrue($declaration->isDateBased(), 'Testing date-based.');
    $this->assertEquals($this->user, $declaration->getDonor()->getContext(), 'Testing declarer.');
  }

  /**
   * Test the end date.
   */
  protected function testEndDate() {
    // End date defaults to indefinite.
    $declaration1 = $this->createDeclaration();
    $this->assertNull($declaration1->getEndDate(), 'Testing end date.');
    $this->assertFalse($declaration1->hasEndDate());
    $declaration1->setEndDate('2050-01-01')->save();
    $this->assertTrue($declaration1->hasEndDate(), "Declaration has an end date");
    $this->assertEquals('2050-01-01', $declaration1->getEndDate(TRUE));
    $declaration1->setEndDate(NULL)->save();
    $this->assertFalse($declaration1->hasEndDate());

    // Creating a declaration with specified end date.
    $declaration2 = $this->createDeclaration(
      ['end_date' => '2050-01-01']
    );
    $this->assertTrue($declaration2->hasEndDate(), "Declaration has an end date");
    $this->assertEquals('2050-01-01', $declaration2->getEndDate(TRUE));
  }

  /**
   *
   */
  protected function assertEndDate($expected, $declaration) {
    $declaration = $this->reloadEntity($declaration);
    $actual = $declaration->hasEndDate() ? $declaration->getEndDate(TRUE) : NULL;
    $this->assertSame($expected, $actual);
  }

  /**
   * Test the declared date.
   */
  public function testDeclaredDate() {
    $declaration = $this->createDeclaration();
    $this->assertNotNull($declaration->getDeclaredDate());
    $declaration->setDeclaredDate('2050-01-01')->save();
    $this->assertEquals('2050-01-01', $declaration->getDeclaredDate(TRUE));

    $declaration = $this->createDeclaration(
      ['declared_date' => '2050-01-01']
    );
    $this->assertEquals('2050-01-01', $declaration->getDeclaredDate(TRUE));
  }

  /**
   * Test the creation of a limited declaration, one that is not date-based.
   */
  public function testSelective() {
    $declaration = $this->createDeclaration([
      'date_based' => FALSE,
      'start_date' => "2022-05-20",
      'claimable_date' => "2022-05-20",
    ]);

    $violations = $declaration->validate();
    $this->assertCount(0, $violations, $this->getViolationMessages($violations));

    $this->assertInstanceOf(Declaration::class, $declaration, 'The declaration should be created.');
    $this->assertFalse($declaration->isDateBased(), "Declaration should not be date-based");
    $this->assertNull($declaration->getStartDate(), 'Start date should be null if declaration is not date-based.');
    $this->assertNull($declaration->getEndDate(), 'Testing end date.');
    $this->assertNull($declaration->getConfirmationDate(), 'Testing written statement sent date.');
    $this->assertEquals($declaration->getStatus(), DeclarationInterface::DECLARATION_SELECTIVE);
  }

  /**
   * Test a start date is required for date-based declarations.
   */
  public function testStartDateRequired() {
    $this->expectException(EntityStorageException::class);
    $declaration = $this->createDeclaration([
      'date_based' => TRUE,
      'start_date' => NULL,
    ]);
  }

  /**
   * Test the creation of declaration with a node as context.
   */
  public function testNodeContext() {
    $node = $this->createNode();

    $declaration = $this->createDeclaration([
      'context' => $node,
    ]);

    $violations = $declaration->validate();
    $this->assertCount(0, $violations, $this->getViolationMessages($violations));
    $declaration = Declaration::load($declaration->id());

    $this->assertInstanceOf(Declaration::class, $declaration, 'The declaration should be created.');
    $this->assertEquals($node->id(), $declaration->getDonor()->getContext()->id(), 'Testing node context.');
  }

  /**
   * Test the declaration status.
   */
  public function testStatus() {
    $declaration = $this->createDeclaration();
    $pastDate = new DrupalDateTime('now -3 day');
    $pastDate = $pastDate->format(DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $futureDate = new DrupalDateTime('now +3 day');
    $futureDate = $futureDate->format(DateTimeItemInterface::DATE_STORAGE_FORMAT);
    $declaration->setClaimableDate($pastDate)->save();

    // Declaration starts today by default.
    $this->assertEquals(DeclarationInterface::DECLARATION_ONGOING, $declaration->getStatus(), 'Declaration should be ongoing by default.');

    $declaration->setStartDate($pastDate);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_ONGOING, $declaration->getStatus(), 'Declaration should be ongoing if start date is in the past.');

    $declaration->setStartDate($futureDate);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_PENDING, $declaration->getStatus(), 'Declaration should be pending if start date is in the future.');

    $declaration->setStartDate($pastDate);
    $declaration->setEndDate($pastDate);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_ENDED, $declaration->getStatus(), 'Declaration should be ended if end date is in the past.');

    $declaration->setDateBased(FALSE);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_SELECTIVE, $declaration->getStatus(), 'Declaration should be selective if not date-based.');

    $declaration->setClaimableDate($futureDate);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_PROVISIONAL, $declaration->getStatus(), 'Declaration should be provisional.');

    $declaration->setValidity(DeclarationInterface::DECLARATION_INVALID);
    $declaration->save();
    $this->assertEquals(DeclarationInterface::DECLARATION_INVALID, $declaration->getStatus(), 'Declaration should be invalid.');
  }

  /**
   *
   */
  protected function getViolationMessages($violations) {
    $messages = [];
    if ($violations->count() > 0) {
      for ($i = 0; $i < $violations->count(); $i++) {
        $violation = $violations->get($i);
        $messages[] = [$violation->getPropertyPath() => (string) $violation->getMessage()];
      }
    }
    return print_r($messages, TRUE);
  }

  /**
   */
  public function testAccess() {
    // Fields that can be set when creating a declaration.
    $make_fields = ['start_date', 'end_date', 'date_based'];
    // Fields only set by staff.
    $staff_fields = [
      'validity',
      'donor',
      'charity',
      'confirmation_date',
    ];

    $admin = $this->createUser(['administer gift aid declarations'])->setUsername('Admin');
    $staff = $this->createUser(['cancel gift aid declarations made by others', 'record gift aid declarations made by others', 'view gift aid declarations'])->setUsername('Staff');
    $donor = $this->createUser(['cancel gift aid declaration', 'make gift aid declaration'])->setUsername('Donor');
    $nobody = $this->createUser()->setUsername('Nobody');

    // When creating a declaration, the only restriction is that
    // donors cannot access staff fields.
    $gift_aid_donor = $this->createDonor()->setContext($donor);
    $declaration = Declaration::create();
    $declaration->setDonor($gift_aid_donor);
    $declaration->setEvidence($this->evidence);
    $declaration->setExplanation('I am a UK tax payer');
    $this->assertFalse($declaration->isClaimable());
    $this->assertTrue($declaration->access('create', $admin));
    $this->assertTrue($declaration->access('create', $staff));
    $this->assertFalse($declaration->access('create', $donor));
    $this->assertFalse($declaration->access('create', $nobody));
    $this->assertFieldAccess($make_fields, $admin, TRUE, $declaration);
    $this->assertFieldAccess($staff_fields, $admin, TRUE, $declaration);
    $this->assertFieldAccess($make_fields, $staff, TRUE, $declaration);
    $this->assertFieldAccess($staff_fields, $staff, TRUE, $declaration);
    $this->assertFieldAccess($make_fields, $donor, TRUE, $declaration);
    $this->assertFieldAccess($staff_fields, $donor, FALSE, $declaration);

    // Once a declaration is made, donors can only cancel it.
    $declaration->save();
    $this->assertTrue($declaration->access('update', $admin));
    $this->assertTrue($declaration->access('update', $staff));
    $this->assertFalse($declaration->access('update', $donor));
    $this->assertFalse($declaration->access('update', $nobody));

    // Staff and admins can view declarations.
    $this->assertTrue($declaration->access('view', $admin));
    $this->assertTrue($declaration->access('view', $staff));
    $this->assertFalse($declaration->access('view', $donor));
    $this->assertFalse($declaration->access('view', $nobody));

    // Staff can edit any field on a recently created declaration.
    $this->assertFieldAccess($make_fields, $staff, TRUE, $declaration);
    $this->assertFieldAccess($staff_fields, $staff, TRUE, $declaration);
    $this->assertFieldAccess($make_fields, $admin, TRUE, $declaration);
    $this->assertFieldAccess($staff_fields, $admin, TRUE, $declaration);

    // Staff cannot edit a claimable declaration, but admins can.
    $declaration->setClaimableDate(new DrupalDateTime('2 months ago'))->save();
    \Drupal::entityTypeManager()->getAccessControlHandler('gift_aid_declaration')->resetCache();
    $this->assertTrue($declaration->isClaimable());
    $this->assertTrue($declaration->access('update', $admin));
    $this->assertFalse($declaration->access('update', $staff));

    // Only admins can delete declarations.
    $this->assertTrue($declaration->access('delete', $admin));
    $this->assertFalse($declaration->access('delete', $staff));
    $this->assertFalse($declaration->access('delete', $donor));
    $this->assertFalse($declaration->access('delete', $nobody));
  }

  /**
   * Checks field access.
   *
   * @param string[] $fields
   *   Array of field names to check.
   * @param \Drupal\user\UserInterface $user
   *   User to check access for.
   * @param bool $access
   *   Expected access.
   * @param \Drupal\gift_aid\Entity\DeclarationInterface $declaration
   *   Declaration to check.
   */
  protected function assertFieldAccess(array $fields, UserInterface $user, bool $access, DeclarationInterface $declaration) {
    $access_text = $access ? 'true' : 'false';
    $user_name = $user->getDisplayName();
    foreach ($fields as $field) {
      $this->assertEquals($access, $declaration->get($field)->access('edit', $user), "Access should be $access_text for field $field and user $user_name");
    }
  }

}
