<?php

namespace Drupal\Tests\gift_aid\Traits;

use Drupal\gift_aid\Entity\Charity;
use Drupal\gift_aid\Entity\Donor;
use Drupal\gift_aid\Entity\Declaration;
use Drupal\gift_aid\Entity\DeclarationInterface;
use Drupal\gift_aid\Entity\Evidence;
use Drupal\gift_aid\Entity\EvidenceType;

/**
 * Helper methods for gift aid tests.
 */
trait GiftAidTestTrait {

  /**
   * A sample user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * A sample donor.
   *
   * @var \Drupal\gift_aid\Entity\DonorInterface
   */
  protected $donor;

  /**
   * A sample charity.
   *
   * @var \Drupal\gift_aid\Entity\CharityInterface
   */
  protected $charity;

  /**
   * A sample evidence type.
   *
   * @var \Drupal\gift_aid\Entity\EvidenceTypeInterface
   */
  protected $evidenceType;

  /**
   * Sample evidence.
   *
   * @var \Drupal\gift_aid\Entity\EvidenceInterface
   */
  protected $evidence;

  /**
   * Create default entities.
   */
  protected function createGiftAidDefaultEntities() {
    $user = $this->createUser(['administer gift aid declarations'])->setEmail('declarationtest@example.com');
    $this->user = $this->reloadEntity($user);
    $this->charity = $this->createCharity(['is_default' => TRUE]);
    $this->evidenceType = $this->createEvidenceType();
    $this->evidence = $this->createEvidence();
    $this->donor = $this->createDonor();
  }

  /**
   * Create a charity.
   *
   * @return \Drupal\gift_aid\Entity\CharityInterface
   *   A new Charity entity.
   */
  public function createCharity($values = []) {
    $name = $this->randomMachineName();
    $defaults = [
      'name' => 'Charity name ' . $name,
      'registered_name' => 'Charity registered name ' . $name,
      'registration_number' => 'Charity registration number ' . $name,
      'registered_address' => 'Charity registered address ' . $name,
    ];
    $charity = Charity::create($values + $defaults);
    $charity->save();
    return Charity::load($charity->id());
  }

  /**
   * Create a donor.
   *
   * @return \Drupal\gift_aid\Entity\DonorInterface
   *   A donor entity.
   */
  protected function createDonor($values = [], $save = TRUE) {
    $defaults = [
      'context' => [
        'target_id' => $this->user->id(),
        'target_type' => 'user',
      ],
      'address' => [
        'country_code' => 'GB',
        'locality' => 'Birmingham',
        'postal_code' => 'B11 1AA',
        'address_line1' => 'My house',
        'given_name' => 'John',
        'family_name' => 'Smith',
      ],
    ];
    $values = $values + $defaults;
    $donor = Donor::create($values);
    if ($save) {
      $donor->save();
    }
    return $donor;
  }

  /**
   * Create an evidence type.
   *
   * @return \Drupal\gift_aid\Entity\EvidenceTypeInterface
   *   A new EvidenceType entity.
   */
  public function createEvidenceType() {
    $name = $this->randomMachineName();
    $evidence_type = EvidenceType::create([
      'id' => 'evidence_type_' . $name,
      'label' => 'Evidence type ' . $name,
      'written_record_required' => FALSE,
    ]);
    $evidence_type->save();
    return EvidenceType::load($evidence_type->id());
  }

  /**
   * Create evidence.
   *
   * @return \Drupal\gift_aid\Entity\EvidenceInterface
   *   A new Evidence entity.
   */
  public function createEvidence() {
    $evidence = Evidence::create([
      'type' => $this->evidenceType->id(),
    ]);
    $evidence->save();
    return Evidence::load($evidence->id());
  }

  /**
   * Create a declaration.
   *
   * @return \Drupal\gift_aid\Entity\DeclarationInterface
   *   A declaration entity.
   */
  protected function createDeclaration($values = []) {
    $defaults = [
      'charity' => $this->charity->id(),
      'donor' => $this->donor->id(),
      'evidence' => $this->evidence->id(),
      'explanation' => 'I am a UK taxpayer and understand my liabilities.',
      'validity' => DeclarationInterface::DECLARATION_VALID,
    ];
    $values = $values + $defaults;
    $declaration = Declaration::create($values);
    $declaration->save();
    return $declaration;
  }

  /**
   *
   */
  protected function getViolationMessages($violations) {
    $messages = [];
    if ($violations->count() > 0) {
      for ($i = 0; $i < $violations->count(); $i++) {
        $violation = $violations->get($i);
        $messages[] = [$violation->getPropertyPath() => (string) $violation->getMessage()];
      }
    }
    return print_r($messages, TRUE);
  }

}
